/*
 * Copyright (c) 2010
 *      Nakata, Maho
 *      All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <complex>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>

#ifdef __cplusplus
extern "C" {
#endif

#define F77_FUNC(name,NAME) name ## _ 
#define dgemm_f77 F77_FUNC (dgemm, DGEMM)
#define F77_RET_I int

F77_RET_I dgemm_f77(const char *transa, const char *transb, int *m, int *n, int *k, double *alpha, double *a, int *lda, double *b, int *ldb, double *beta, double *c, int *ldc);

#ifdef __cplusplus
}
#endif

#define MAXLOOP 10

unsigned long long microseconds()
{
    rusage  t;
    timeval tv;
    getrusage( RUSAGE_SELF, &t );
    tv = t.ru_utime;
    return ((unsigned long long)tv.tv_sec)*1000000 + tv.tv_usec;
}

double gettimeofday_sec()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec + (double)tv.tv_usec*1e-6;
}

int
main()
{
    int n;
    int incx = 1, incy = 1;
    double alpha = 3.14, beta = 2.717;
    double dgemmtime, t1, t2, t_1, t_2;

     for (n = 3000 ; n < 10000; n=n+100) {
	printf("n: %d\n", (int)n);
	double *A = new double[n*n];
	double *B = new double[n*n];
	double *C = new double[n*n];
	for (int i = 0; i < n; i++) {
	  for (int j = 0; j < n; j++) {
	    A[i*n+j] = i * j + 1;
	    B[i*n+j] = (i+1) * (j+1) + 1;
	    C[i*n+j] = (i+1) - (j+1) + 1;
	  }
	}
    t1 = (double)microseconds(); t_1 = gettimeofday_sec();
      for (int p = 0 ; p < MAXLOOP; p++ ){
 	   dgemm_f77("n", "n", &n, &n, &n, &alpha, A, &n, B, &n, &beta, C, &n);
      }
    t2 = (double)microseconds(); t_2 = gettimeofday_sec();
 //	dgemmtime = (t2 - t1) * 1e-6;
 	dgemmtime = (t_2 - t_1);
	printf("time : %lf or %lf \n", (t2 - t1) * 1e-6, t_2 - t_1);
 	printf("Mflops : %lf\n", ( 2.0 * (double)n * (double)n * (double)n + 2.0 * (double)n* (double)n )* MAXLOOP / dgemmtime / (1000*1000) );
	delete[]C;
	delete[]B;
	delete[]A;
    }
}
